#!/bin/bash

if [ "$1" == "-h" ] || [ "$1" == "--help" ]; then
   echo
   echo 'Installer script for the mvBlueCOUGAR driver.'
   echo
   echo " Usage : $0 [directory_name] [version]"
   echo
   echo "Optional argument 'directory_name' specifies where to place the driver."
   echo 'If the directory does not yet exist, it will be created.'
   echo
   echo " Example : $0 $HOME/matrix-vision/mvBlueCOUGAR"
   echo
   echo "Optional argument 'version' specifies which .tgz file to unpack."
   echo 'This is only relevant when the archive directory contains multiple'
   echo 'versions of mvBlueCOUGAR-n.n.n.n.tgz.'
   echo "If this is so and 'version' is not specified on the command line, the"
   echo 'latest version will be unpacked.'
   echo
   echo 'To install in the current working directory, simply leave out the'
   echo "directory name, or specify '.' So the two commands below do the same:"
   echo
   echo " $0 1.10.33.111"
   echo " $0 . 1.10.33.111"
   echo
   echo "To summarize: 'directory_name' defaults to the current working directory"
   echo "and 'version' defaults to the latest version. Consequently, if no"
   echo 'arguments are passed to the script, the latest version is installed'
   echo 'in the current working directory.'
   echo
   echo 'This script must be in the directory that contains mvBlueCOUGAR-n.n.n.n.tgz'
   echo
   exit
fi

if [ "$1" == "." ] || [ "$1" == "./" ]; then
   set $PWD $2
elif ! $(echo $1 | grep -q [a-zA-Z]); then
   # Nothing or just the version (e.g. 1.10.30.108) was specified
   # on the command line, i.e. no installation path was specified.
   # Install in the current working directory.
   # Without this test, if just the version was specified, that
   # (e.g. 1.10.30.108) would become the name of the newly created
   # directory, and the version would no longer be explicitly specified.
   # We assume that the directory name will contain at least one character.
   set $PWD $1
   echo
   echo 'No target directory specified'
   echo 'Will use the current working directory'
else
   PLEN=`expr length $1`
   if [ "`expr substr $1 $PLEN $PLEN`" == "/" ]; then
      # strip the trailing / and reassign to positional argument one
      # (which can only be done using 'set')
      PLAST=`expr $PLEN - 1`
      set `expr substr $1 1 $PLAST` $2
   fi

   if [ "`expr substr $1 1 1`" != "/" ]; then
      # make an absolute path out of a relative one and reassign
      # to positional argument one (which can only be done using 'set')
      set $PWD/$1 $2
   fi
fi

# TESTVERARG=$1
# while $(echo $TESTVERARG | grep -q '\.'); do TESTVERARG=`echo $TESTVERARG | sed -e s/\\\.//`;done

LIBVER=$2

echo
if which dirname >/dev/null; then
    SCRIPTSOURCEDIR=$(dirname $0)
fi
if [ "$SCRIPTSOURCEDIR" != "$PWD" ]; then
   if [ -z $SCRIPTSOURCEDIR ] || [ "$SCRIPTSOURCEDIR" == "." ]; then
      SCRIPTSOURCEDIR=$PWD
   else
      cd $SCRIPTSOURCEDIR
      if [ "$SCRIPTSOURCEDIR" != "$PWD" ]; then
         echo "Could not cd to sourcedir"
         echo
      fi
   fi
fi

if grep -q '/etc/ld.so.conf.d/' /etc/ld.so.conf; then
   GENICAM_LDSOCONF_FILE=/etc/ld.so.conf.d/genicam.conf
   ACQUIRE_LDSOCONF_FILE=/etc/ld.so.conf.d/acquire.conf
   sudo rm -f $GENICAM_LDSOCONF_FILE; sudo touch $GENICAM_LDSOCONF_FILE
   sudo rm -f $ACQUIRE_LDSOCONF_FILE; sudo touch $ACQUIRE_LDSOCONF_FILE
else
   GENICAM_LDSOCONF_FILE=/etc/ld.so.conf
   ACQUIRE_LDSOCONF_FILE=/etc/ld.so.conf
fi

# make sure the environment variables are set at the next boot as well
if grep -q '/etc/profile.d/' /etc/profile; then
   GENICAM_EXPORT_FILE=/etc/profile.d/genicam.sh
   ACQUIRE_EXPORT_FILE=/etc/profile.d/acquire.sh
   sudo rm -f $GENICAM_EXPORT_FILE; sudo touch $GENICAM_EXPORT_FILE
   sudo rm -f $ACQUIRE_EXPORT_FILE; sudo touch $ACQUIRE_EXPORT_FILE
else
   GENICAM_EXPORT_FILE=/etc/profile
   ACQUIRE_EXPORT_FILE=/etc/profile
fi

if ! [ -r $GENICAM_EXPORT_FILE ]; then
   echo 'Error : cannot write to' $GENICAM_EXPORT_FILE.
   echo 'After the next boot, the required environment variables will not be set.'
   echo
else
   # tests below do not yet check for *commented out* export lines
   if grep -q 'GENICAM_ROOT=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_ROOT already defined in' $GENICAM_EXPORT_FILE.
   else
      sudo sh -c "echo 'export GENICAM_ROOT=$1' >> $GENICAM_EXPORT_FILE"
   fi
   if grep -q 'GENICAM_ROOT_V2_0=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_ROOT_V2_0 already defined in' $GENICAM_EXPORT_FILE.
   else
      sudo sh -c "echo 'export GENICAM_ROOT_V2_0=$1' >> $GENICAM_EXPORT_FILE"
   fi
   if grep -q 'GENICAM_CACHE_V2_0=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_CACHE_V2_0 already defined in' $GENICAM_EXPORT_FILE.
   else
      sudo sh -c "echo 'export GENICAM_CACHE_V2_0=/tmp' >> $GENICAM_EXPORT_FILE"
   fi
   if grep -q 'GENICAM_LOG_CONFIG_V2_0=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_LOG_CONFIG_V2_0 already defined in' $GENICAM_EXPORT_FILE.
   else
      sudo sh -c "echo 'export GENICAM_LOG_CONFIG_V2_0=$1/log/config-unix/DefaultLogging.properties' >> $GENICAM_EXPORT_FILE"
   fi
fi

# move to the destination directory
if ! [ -d $1 ]; then
   # the destination directory does not yet exist
   # first try to create it as a normal user
   mkdir --parent $1 >/dev/null 2>&1
   if ! [ -d $1 ]; then
      # that didn't work
      # now try it as superuser
      sudo mkdir --parent $1
   fi
   if ! [ -d $1 ]; then
      echo 'Error : could not create target directory' $1.
      echo 'Maybe you specified a partition that was mounted read only.'
      echo
      exit
   fi
fi

TARGET=$(uname -m)
if [ x$TARGET = xi686 ]; then
   TARGET=x86
fi

# now unpack the actual libraries
if ! [ -z $LIBVER ] && [ -r mvBlueCOUGAR-$TARGET-$LIBVER.tgz ]; then
   TARFILE=mvBlueCOUGAR-$TARGET-$LIBVER.tgz
else
   for a in mvBlueCOUGAR-$TARGET-*.tgz; do TARFILE=$a; done
fi

if ! [ -r $TARFILE ]; then
   echo 'Error : could not read' $TARFILE.
   echo
   exit
fi

# for tarfile mvBlueCOUGAR-x86-1.10.30.108.tgz, VERSION would be 1.10.30.108
VERSION=`echo $TARFILE | sed -e s/mvBlueCOUGAR-$TARGET-// | sed -e s/\\.tgz//`

# needed at compile time (used during development, but not shipped with the final program)
AC=mvimpact_acquire
ACT=$AC-$VERSION.tar

# needed at run time
BC=mvBlueCOUGAR_runtime
BCT=$BC-$VERSION.tar

if ! [ -z $LIBVER ] && [ "$LIBVER" != "$VERSION" ]; then
   echo 'Warning : requested version' $LIBVER, 'found version' $VERSION.
   echo
fi

cd /tmp
sudo tar xfz $SCRIPTSOURCEDIR/$TARFILE

cd $1
# remove older versions (if any)
rm -f lib/libmv*.so*

echo
sudo tar xvf /tmp/$BCT
echo

# the runtime tar contains either the i86 or the x64 tgz
if [ -r GenICam_Runtime_gcc40_Linux32_i86_v2_0.tgz ]; then
   sudo tar xvfz GenICam_Runtime_gcc40_Linux32_i86_v2_0.tgz;
   if [ x$TARGET != xx86 ]; then
      echo 'Platform conflict : GenICam runtime is 32bit, but target is 64bit'
   fi
fi
if [ -r GenICam_Runtime_gcc40_Linux64_x64_v2_0.tgz ]; then
   sudo tar xvfz GenICam_Runtime_gcc40_Linux64_x64_v2_0.tgz;
   if [ x$TARGET = xx86 ]; then
      echo 'Platform conflict : GenICam runtime is 64bit, but target is 32bit'
   fi
fi

sudo chown -R $(whoami):$(whoami) xml log licenses bin License_ReadMe.txt

# now see if we can unpack the tar file with the device independent stuff
# this is entirely optional
# take care: the user could specify $ACQUIREROOT/mvimpact-acquire as installation path
if [ -r /tmp/$ACT ]; then
   if [ -d /home/mvbc/workspace ]; then
      ACQUIREROOT=/home/mvbc/workspace
   else
      ACQUIREROOT=$HOME
   fi
   cd $ACQUIREROOT
   tar xvf /tmp/$ACT
   if [ "$1" != "$ACQUIREROOT/mvimpact-acquire" ]; then
      # this only removes the softlink
      rm -f mvimpact-acquire
      ln -s $AC-$VERSION mvimpact-acquire

      # mvDeviceConfigure expects directory $MVIMPACT_ACQUIRE_DIR/FirmwareUpdate
      if [ "$1" != "$ACQUIREROOT/$AC-$VERSION" ]; then
         cd mvimpact-acquire
         if ! [ -h FirmwareUpdates ]; then
            ln -s $1/FirmwareUpdates .
         fi
      fi
   else
      cd mvimpact-acquire
      for a in $ACQUIREROOT/$AC-$VERSION/*; do
         if [ "$(basename $a)" != "lib" ]; then
            rm -f $(basename $a)
            ln -s $a .
         fi
      done
      cd lib
      rm -f $TARGET
      ln -s $ACQUIREROOT/$AC-$VERSION/lib/$TARGET .
   fi

   if grep -q 'MVIMPACT_ACQUIRE_DIR=' $ACQUIRE_EXPORT_FILE; then
      echo 'MVIMPACT_ACQUIRE_DIR already defined in' $ACQUIRE_EXPORT_FILE.
   else
      sudo sh -c "echo 'export MVIMPACT_ACQUIRE_DIR=$ACQUIREROOT/mvimpact-acquire' >> $ACQUIRE_EXPORT_FILE"
   fi

   if grep -q "$ACQUIREROOT/mvimpact-acquire/lib/$TARGET" $ACQUIRE_LDSOCONF_FILE; then
      echo "$ACQUIREROOT/mvimpact-acquire/lib/$TARGET already defined in" $ACQUIRE_LDSOCONF_FILE.
   else
      sudo sh -c "echo '$ACQUIREROOT/mvimpact-acquire/lib/$TARGET' >> $ACQUIRE_LDSOCONF_FILE"
   fi
fi

# now do the shared linker setup
if ! [ -r $GENICAM_LDSOCONF_FILE ]; then
   echo 'Error : cannot write to' $GENICAM_LDSOCONF_FILE.
   echo 'Execution will fail, as at run time, the shared objects will not be found.'
   echo
else
   if [ x$TARGET = xx86 ]; then
      GENILIBPATH=Linux32_i86
   else
      GENILIBPATH=Linux64_x64
   fi
   # tests below do not check for *commented out* link lines
   # must later add substring check

   # acquire libs
   if grep -q "$1/lib" $GENICAM_LDSOCONF_FILE; then
      echo "$1/lib already defined in" $GENICAM_LDSOCONF_FILE.
   else
      sudo sh -c "echo '$1/lib' >> $GENICAM_LDSOCONF_FILE"
   fi

   # genicam libs
   if grep -q "$1/bin/$GENILIBPATH" $GENICAM_LDSOCONF_FILE; then
      echo "$1/bin/$GENILIBPATH already defined in" $GENICAM_LDSOCONF_FILE.
   else
      sudo sh -c "echo '$1/bin/$GENILIBPATH' >> $GENICAM_LDSOCONF_FILE"
   fi
   if grep -q "$1/bin/$GENILIBPATH/GenApi/Generic" $GENICAM_LDSOCONF_FILE; then
      echo "$1/bin/$GENILIBPATH/GenApi/Generic already defined in" $GENICAM_LDSOCONF_FILE.
   else
      sudo sh -c "echo '$1/bin/$GENILIBPATH/GenApi/Generic' >> $GENICAM_LDSOCONF_FILE"
   fi
fi

sudo /sbin/ldconfig

if [ -r /tmp/$ACT ]; then
   # the shared libs are part of the runtime tree, but the Makefile expects
   # them in the development tree, so we add links here
   # this must be done after ldconfig, so we can link to simple *.so files
   cd $ACQUIREROOT/$AC-$VERSION/lib/$TARGET
   for a in $1/lib/libmv*.so; do
      ln -f -s $a . >/dev/null 2>&1
   done
fi

rm -f /tmp/$ACT /tmp/$BCT

COULD_NOT_INSTALL="Found neither apt-get nor yast; please install >%s< manually."

# check if we have g++
if ! which g++ >/dev/null 2>&1; then
   if which apt-get >/dev/null 2>&1; then
      sudo apt-get -q install g++
   elif sudo which yast >/dev/null 2>&1; then
      YASTBIN=`sudo which yast`
      sudo $YASTBIN --install gcc-c++
   else
      printf "$COULD_NOT_INSTALL" "g++"
   fi
fi

# check if we have libexpat
if ! ldconfig -p | grep -q "expat"; then
   if which apt-get >/dev/null 2>&1; then
      sudo apt-get -q install libexpat1-dev
   elif sudo which yast >/dev/null 2>&1; then
      YASTBIN=`sudo which yast`
      sudo $YASTBIN --install expat
   else
      printf "$COULD_NOT_INSTALL" "libexpat"
   fi
fi

INPUT_REQUEST="Do you want to install >%s< (default is 'yes')?\nHit 'n' + <Enter> for 'no', or just <Enter> for 'yes'.\n"
YES_NO=

# do we want to install wxWidgets?
if ! which wx-config >/dev/null 2>&1; then
   echo
   printf "$INPUT_REQUEST" "wxWidgets"
   echo "This is highly recommended, as without wxWidgets, you cannot build wxPropView."
   echo
   read YES_NO
   if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
      echo 'Not installing wxWidgets'
   else
      if which apt-get >/dev/null 2>&1; then
         echo 'Installing wxWidgets'
         sudo apt-get -q install libwxgtk2.8-dev
      elif sudo which yast >/dev/null 2>&1; then
         echo 'Installing wxWidgets'
         YASTBIN=`sudo which yast`
         sudo $YASTBIN --install wxGTK-devel
      else
         printf "$COULD_NOT_INSTALL" "wxWidgets"
      fi
   fi
fi

# do we want to install FLTK?
if ! which fltk-config >/dev/null 2>&1; then
   echo
   printf "$INPUT_REQUEST" "FLTK"
   echo "This is only required if you want to build the 'LiveSnapFLTK' sample."
   echo
   read YES_NO
   if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
      echo 'Not installing FLTK'
   else
      if which apt-get >/dev/null 2>&1; then
         echo 'Installing FLTK'
         sudo apt-get -q install libgl1-mesa-dev
         sudo apt-get -q install libfltk1.1-dev
      elif sudo which yast >/dev/null 2>&1; then
         echo 'Installing FLTK'
         YASTBIN=`sudo which yast`
         sudo $YASTBIN --install Mesa-devel
         sudo $YASTBIN --install fltk-devel
      else
         printf "$COULD_NOT_INSTALL" "FLTK"
      fi
   fi
fi

echo
echo "Do you want the tools and samples to be built (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
read YES_NO
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
   echo 'The tools and samples were not built.'
   echo 'To build them yourself, type:'
   echo '  cd ~/mvimpact-acquire'
   echo "  make $TARGET"
   echo '  sudo /sbin/ldconfig'
else
   cd $ACQUIREROOT/mvimpact-acquire
   make $TARGET
   sudo /sbin/ldconfig
fi

echo
echo 'Now execute the following four commands:'
echo '  ' export GENICAM_ROOT=$1
echo '  ' export GENICAM_ROOT_V2_0=$1
echo '  ' export GENICAM_CACHE_V2_0=/tmp
echo '  ' export GENICAM_LOG_CONFIG_V2_0=$1/log/config-unix/DefaultLogging.properties
echo 'or source' $GENICAM_EXPORT_FILE:
echo '  ' . $GENICAM_EXPORT_FILE
echo 'or restart X11 (Ctrl-Alt-Backspace (twice on OpenSuSE)).'
echo '(The four exports above have been added to' $GENICAM_EXPORT_FILE.
echo ' This file is read when you log in, which is normally done by starting X11.)'
echo
